#include "mbed.h"
#include <string.h>
#include <string>
#include <ctype.h>
#include "define.h"
#include "BlockDevice.h"
#include "FATFileSystem.h"

#define MAX_LEN 4
#define SZRING  128
#define CTRLC   0x03
#define LF      0x0a
#define CR      0x0d
#define BS      0x08

class Serial{
private:
	UnbufferedSerial sp;
public:
	Serial(PinName, PinName);
	void baud(int);
	bool readable(void);
	bool writeable(void);
	char getc(void);
	void putc(char);
	void puts(const char*);
	void printf(const char*);
};

extern Serial pc;
extern void Z80Init();
extern void Z80Run();
extern uint8 Z80Memory[];
extern uint8 ReadMemory(uint16);
extern void WriteMemory(uint16, uint8);
extern short RecvZ80();
extern DigitalOut myled;
extern uint16 PC;

/*-----------------------------------------------------------------------2014.4.28
const uint8 rom[] = {
  0x31,0x00,0x10,0xcd,0x37,0x20,0x06,0x37,0x11,0x01,0x00,0x21,0x00,0xdc,0x7a,0xd3,
  0xa0,0xd3,0xa1,0x7b,0xd3,0xa2,0x7d,0xd3,0xa4,0x7c,0xd3,0xa5,0x3e,0x01,0xd3,0xa6,
  0xc5,0x01,0x80,0x00,0x09,0xc1,0x1c,0x7b,0xfe,0x40,0xc2,0x30,0x20,0x14,0x1e,0x00,
  0x05,0xc2,0x0e,0x20,0xc3,0x33,0xf2,0x3e,0x69,0xd3,0x82,0x3e,0x70,0xd3,0x82,0x3e,
  0x6c,0xd3,0x82,0x3e,0x0d,0xd3,0x82,0x3e,0x0a,0xd3,0x82,0xc9
};
--------------------------------------------------------------------------------*/

uint16 dma      = 0;
uint16 dma_l    = 0;
uint16 dma_h    = 0;
uint32 sector   = 0;
uint32 track    = 0;
uint32 track_l  = 0;
uint32 track_h  = 0;
uint32 headpos  = 0;
//int    disk_err = 0;
uint8    disk_err = 0;

//DigitalOut cs(PB_6);
//SPI spi(PA_7, PA_6, PA_5); // mosi, miso, sclk    //2014.4.21

FileSystem *FileSystem::get_default_instance()
{
    static FATFileSystem default_fs("sd", BlockDevice::get_default_instance());
    return &default_fs;
}

BlockDevice *bd = BlockDevice::get_default_instance();

FATFileSystem sd("sd");

FILE *fp;
/******************************/
/* c_getch                    */
/******************************/
uint8 c_getch() {
    while(!pc.readable());
    return pc.getc();
}
/******************************/
/* setup                      */
/******************************/
void setup(void) {
  bool err = false;
  int er=1;

  while(er) {
      pc.printf("Mounting SD card...");
      er = sd.mount(bd);
      if(er) {pc.printf("ERROR\n\r");
           sd.unmount();
           while(c_getch()!= CR);
      }
  }
  pc.printf("OK.\r\npress ENTER key.");

  char DIRPATH[]="/sd";
  DIR *dir;
  struct dirent *entry;

 // cs = 1;
  //pc.baud( BPS );
  myled = 0;

  while(c_getch()!= CR);
  pc.printf("\r\ncpm80.dsk..");
  dir = opendir(DIRPATH);
  if ( dir != NULL ) {
    while ( (entry = readdir(dir)) != NULL ) {
    //  if(strcmp (entry->d_name, "cpm80.dsk")==0) {
        if(entry->d_name==std::string("cpm80.dsk")) {
          pc.printf("ok\r\n");
          err = true;
          break;
        }
    }
  }
  closedir(dir);

  fp = fopen("/sd/cpm80.dsk", "r+b");   //2014.4.28
  if(fp == NULL) {
    pc.puts("Could not open file for read/write\r\n");
    err = true;
  }
  
  if(!err) {
    pc.printf("error!\r\n");
    while(true);
  }

  // ipl
  /*------------------------------
  for (uint16 i=0; i<128; i++) {
    WriteMemory(GOIPL+i, rom[i]);
  }
--------------------------------*/
  void cpmdsk_seek(uint32);     //2014.4.20
  void cpmdsk_read(void);
  dma = GOIPL;
  cpmdsk_seek(0);
  cpmdsk_read();
  
  Z80Init();
  PC = GOIPL;
}
/******************************/
/* loop                       */
/******************************/
void loop(void) {
}
/******************************/
/* cpmdsk_seek                */
/******************************/
void cpmdsk_seek(uint32 seekp) {
/*---------------------------------------------2014.4.28
  fp = fopen("/sd/cpm80.dsk", "r+b");
  if(fp == NULL) {
    pc.puts("Could not open file for read/write\r\n");
    return;
  }
  ----------------------------------------------------*/
  fseek(fp, seekp, SEEK_SET);
}
/******************************/
/* cpmdsk_read                */
/******************************/
void cpmdsk_read(void) {
  char buf[128];
  myled = 1;
  if (fread(buf, sizeof(buf), 1, fp) == 0) {
      pc.printf("fread error!\r\n");
      fclose(fp);
      return;
  }
  for (uint16 i=0; i<128; i++) {
    WriteMemory(dma+i, buf[i]);
  }
  //fclose(fp);
  myled = 0;
}
/******************************/
/* cpmdsk_write               */
/******************************/
void cpmdsk_write(void) {
  char buf[128];
  myled = 1;
  for (uint16 i=0; i<128; i++) {
    buf[i] = ReadMemory(dma + i);
  }
  if(fwrite(buf, sizeof(buf), 1, fp) == 0) {
      pc.printf("fwrite error!\r\n");
  }
  //fclose(fp);
  myled = 0;
}
